{% extends "base.html" %}
{% block title %}
    About
{% endblock %}

{% block body %}

<div class="text-center bg-body-tertiary">
    <div class="container py-5">
      <h1 class="text-body-emphasis">What is LavaCactus</h1>
      <p class="col-lg-8 mx-auto lead">
        LavaCactus is a simple but powerful static website generator using
          Python and the <a href="https://docs.djangoproject.com/en/dev/topics/templates/" target="_blank">Django template system</a>. LavaCactus also makes it easy to develop
          locally and deploy your site to S3 directly. It works great for company,
          portfolio, personal, support websites and blogs.
      </p>
        <h2 class="text-body-emphasis">Examples</h2>
      <p class="col-lg-8 mx-auto lead">
        <ul class="list-group">
            <li class="list-group-item"><a href="https://findsimilar.org" target="_blank">findsimilar.org</a></li>
            <li class="list-group-item"><a href="https://lavacactus.craftsman.lol" target="_blank">This project website</a></li>
            <li class="list-group-item"><a href="https://craftsman.lol" target="_blank">quill craftsman about website</a></li>
        </ul>
      </p>
        <h2 class="text-body-emphasis">Project Status</h2>
        <p class="col-lg-8 mx-auto lead">
            This is Open Source project with BSD-3-Clause license
        </p>
    </div>
  </div>
{% endblock %}
