{% extends "base.html" %}
{% block title %}
    LavaCactus
{% endblock %}

{% block body %}

<div class="text-center bg-body-tertiary rounded-3">
    <img src="static/images/logo.png" class="bi mt-4 mb-3" width="100" height="100">
    <h1 class="text-body-emphasis">Welcome to LavaCactus!</h1>
    <p class="col-lg-8 mx-auto fs-5 text-muted">
      Get started with this template based on Bootstrap 5. Go ahead, explore!
    </p>
    <div class="d-inline-flex gap-2 mb-5">
        <a class="d-inline-flex align-items-center btn btn-primary btn-lg px-4 rounded-pill" href="https://github.com/quillcraftsman/lavacactus/" target="_blank">Project on GitHub</a>
      <a class="btn btn-outline-secondary btn-lg px-4 rounded-pill" href="contact.html">Contact</a>
    </div>
  </div>


<div class="container">
  <!-- Example row of columns -->
  <div class="row">

    <div class="col-lg-3">
      <h2>Full featured template engine</h2>
      <p>LavaCactus uses and extends Django's templating engine. If you've built sites using Django before, you'll feel right at home.</p>
    </div>

    <div class="col-lg-3">
      <h2>Pretty URLs</h2>
      <p>LavaCactus can prettify URLs, so you can turn <code>/about.html</code> into a fancier <code>/about/</code> or just use <code>about.html</code>.</p>
    </div>

    <div class="col-lg-3">
      <h2>Minification & Fingerprinting</h2>
      <p>For maximum performance, LavaCactus can minify and fingerprint your static assets. As for your pages, LavaCactus will automatically compress them.</p>
   </div>

    <div class="col-lg-3">
      <h2>And much more</h2>
      <p>There's of course a lot more you can do with LavaCactus. Plus, LavaCactus's plugin framework lets you easily extend its functionality.</p>
   </div>


  </div>
</div>
{% endblock %}
