{% extends "base.html" %}
{% block title %}
    Contact
{% endblock %}

{% block body %}

<div class="text-center bg-body-tertiary rounded-3">
    <img src="static/images/logo.png" class="bi mt-4 mb-3" width="100" height="100">
   <h1 class="text-body-emphasis">Contact</h1>
    <p class="col-lg-8 mx-auto fs-5 text-muted">
       <ul class="list-group">
            <li class="list-group-item"><a href="https://github.com/quillcraftsman/lavacactus/" target="_blank">Project on Github</a></li>
            <li class="list-group-item"><a href="mailto:quill@craftsman.lol">quill@craftsman.lol</a></li>
        </ul>
    </p>
  </div>
{% endblock %}
