<!DOCTYPE html>
<html class="no-js">
    <head>
        <meta charset="utf-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
        <title>
            {% block title %}
                LavaCactus
            {% endblock %}
        </title>
        <meta name="description" content="">
        <meta name="viewport" content="width=device-width">
        <link rel="icon" type="image/x-icon" href="static/images/favicon.ico" />
        <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap-icons@1.10.5/font/bootstrap-icons.css">
        <link rel="stylesheet" href="static/css/bootstrap.css">
    </head>

    <body>
    <nav class="navbar navbar-expand-md navbar-dark bg-dark mb-4">
  <div class="container-fluid">
    <a class="navbar-brand" href="index.html">LavaCactus</a>
    <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarCollapse" aria-controls="navbarCollapse" aria-expanded="false" aria-label="Toggle navigation">
      <span class="navbar-toggler-icon"></span>
    </button>
    <div class="collapse navbar-collapse" id="navbarCollapse">
      <ul class="navbar-nav me-auto mb-2 mb-md-0">
       <li class="nav-item">
                <a class="nav-link {% if_current_page 'index.html' 'active' '' %}" {% if_current_page 'aria-current="page"' '' %} href="index.html">Home</a>
            </li>
            <li class="nav-item">
                <a class="nav-link {% if_current_page 'about.html' 'active' '' %}" {% if_current_page 'aria-current="page"' '' %} href="about.html">About</a>
            </li>
            <li class="nav-item">
                <a class="nav-link {% if_current_page 'contact.html' 'active' '' %}" {% if_current_page 'aria-current="page"' '' %} href="contact.html">Contact</a>
            </li>
      </ul>
    </div>
  </div>
</nav>

    {% block body %}
    {% endblock %}

    <footer class="d-flex flex-wrap justify-content-between align-items-center py-3 my-4 border-top">
    <div class="col-md-4 d-flex align-items-center">
      <a href="index.html" class="mb-3 me-2 mb-md-0 text-muted text-decoration-none lh-1">
        <img src="static/images/logo.png" class="bi" width="30" height="24">
      </a>
      <span class="mb-3 mb-md-0 text-muted">Copyleft &#127279; LavaCactus 2023</span>
    </div>

        <ul class="nav col-md-4 justify-content-end">
      <li class="nav-item"><a href="https://lavacactus.craftsman.lol" target="_blank" class="nav-link px-2 text-muted">This site was created by LavaCactus</a></li>
            <li class="nav-item"><a class="nav-link px-2 text-muted" target="_blank" href="https://github.com/quillcraftsman/lavacactus"><i class="bi bi-github"></i></a></li>
    </ul>

  </footer>

    <script src="{% static 'static/js/vendor/jquery-3.7.0.js' %}"></script>
    <script src="{% static 'static/js/vendor/bootstrap.js' %}"></script>
  </body>

  {% block scripts %}
  {% endblock %}
</html>
