<!doctype html>
<html>
<head>
	<meta charset="utf-8">
	<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
	<meta name="viewport" content="width=device-width">

	<link rel="shortcut icon" href="{% static '/static/images/favicon.ico' %}">
	<link rel="stylesheet" href="{% static '/static/css/style.css' %}">

	{% block header %}
	<title>Welcome</title>
	{% endblock %}

</head>
<body>
	{% block content %}
	Main content
	{% endblock %}

	{% block scripts %}
	<script src="//ajax.googleapis.com/ajax/libs/jquery/1.7.1/jquery.min.js"></script>
	<script src="{% static '/static/js/main.js' %}"></script>
	{% endblock %}
</body>
</html>
